@@ -193,20 +193,81 @@ public:
                 pointerMoveEvent.setSerial(LTime::nextSerial());
                 pointerMoveEvent.notify();
                 break;
+            // Legacy wheel scroll events
+            case LIBINPUT_EVENT_POINTER_AXIS:
+                dev = libinput_event_get_device(ev);
+                inputDevice = (LInputDevice*)libinput_device_get_user_data(dev);
+                pointerEvent = libinput_event_get_pointer_event(ev);
+                pointerScrollEvent.setDevice(inputDevice);
+                pointerScrollEvent.setHasX(libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL) != 0);
+                pointerScrollEvent.setHasY(libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL) != 0);
+                pointerScrollEvent.setMs(libinput_event_pointer_get_time(pointerEvent));
+                pointerScrollEvent.setUs(libinput_event_pointer_get_time_usec(pointerEvent));
+                pointerScrollEvent.setSerial(LTime::nextSerial());
+
+                if (libinput_event_pointer_get_axis_source(pointerEvent) == LIBINPUT_POINTER_AXIS_SOURCE_WHEEL)
+                {
+                    if (pointerScrollEvent.hasX())
+                    {
+                        pointerScrollEvent.setX(libinput_event_pointer_get_axis_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL));
+                        pointerScrollEvent.setDiscreteX(libinput_event_pointer_get_axis_value_discrete(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL));
+                    }
+                    else
+                    {
+                        pointerScrollEvent.setX(0.f);
+                        pointerScrollEvent.setDiscreteX(0);
+                    }
+
+                    if (pointerScrollEvent.hasY())
+                    {
+                        pointerScrollEvent.setY(libinput_event_pointer_get_axis_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL));
+                        pointerScrollEvent.setDiscreteY(libinput_event_pointer_get_axis_value_discrete(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL));
+                    }
+                    else
+                    {
+                        pointerScrollEvent.setY(0.f);
+                        pointerScrollEvent.setDiscreteY(0);
+                    }
+
+                    pointerScrollEvent.setSource(LPointerScrollEvent::WheelLegacy);
+                    pointerScrollEvent.notify();
+                }
+                else if (libinput_event_pointer_get_axis_source(pointerEvent) == LIBINPUT_POINTER_AXIS_SOURCE_WHEEL_TILT)
+                {
+                    if (pointerScrollEvent.hasX())
+                        pointerScrollEvent.setX(libinput_event_pointer_get_axis_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL));
+                    else
+                        pointerScrollEvent.setX(0.f);
+
+                    if (pointerScrollEvent.hasY())
+                        pointerScrollEvent.setY(libinput_event_pointer_get_axis_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL));
+                    else
+                        pointerScrollEvent.setY(0.f);
+
+                    pointerScrollEvent.setDiscreteAxes(0, 0);
+                    pointerScrollEvent.setSource(LPointerScrollEvent::WheelTilt);
+                    pointerScrollEvent.notify();
+                }
+                break;
             case LIBINPUT_EVENT_POINTER_SCROLL_FINGER:
                 dev = libinput_event_get_device(ev);
                 inputDevice = (LInputDevice*)libinput_device_get_user_data(dev);
                 pointerEvent = libinput_event_get_pointer_event(ev);
                 pointerScrollEvent.setDevice(inputDevice);
+                pointerScrollEvent.setHasX(libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL) != 0);
+                pointerScrollEvent.setHasY(libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL) != 0);
 
-                if (libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL))
+                if (pointerScrollEvent.hasX())
                     pointerScrollEvent.setX(libinput_event_pointer_get_scroll_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL));
+                else
+                    pointerScrollEvent.setX(0.f);
 
-                if (libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL))
+                if (pointerScrollEvent.hasY())
                     pointerScrollEvent.setY(libinput_event_pointer_get_scroll_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL));
+                else
+                    pointerScrollEvent.setY(0.f);
 
-                pointerScrollEvent.set120X(0.f);
-                pointerScrollEvent.set120Y(0.f);
+                pointerScrollEvent.setDiscreteAxes(0, 0);
                 pointerScrollEvent.setSource(LPointerScrollEvent::Finger);
                 pointerScrollEvent.setMs(libinput_event_pointer_get_time(pointerEvent));
                 pointerScrollEvent.setUs(libinput_event_pointer_get_time_usec(pointerEvent));
@@ -219,14 +280,20 @@ public:
                 pointerEvent = libinput_event_get_pointer_event(ev);
                 pointerScrollEvent.setDevice(inputDevice);
 
-                if (libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL))
+                pointerScrollEvent.setHasX(libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL) != 0);
+                pointerScrollEvent.setHasY(libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL) != 0);
+
+                if (pointerScrollEvent.hasX())
                     pointerScrollEvent.setX(libinput_event_pointer_get_scroll_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL));
+                else
+                    pointerScrollEvent.setX(0.f);
 
-                if (libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL))
+                if (pointerScrollEvent.hasY())
                     pointerScrollEvent.setY(libinput_event_pointer_get_scroll_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL));
+                else
+                    pointerScrollEvent.setY(0.f);
 
-                pointerScrollEvent.set120X(0.f);
-                pointerScrollEvent.set120Y(0.f);
+                pointerScrollEvent.setDiscreteAxes(0, 0);
                 pointerScrollEvent.setSource(LPointerScrollEvent::Continuous);
                 pointerScrollEvent.setMs(libinput_event_pointer_get_time(pointerEvent));
                 pointerScrollEvent.setUs(libinput_event_pointer_get_time_usec(pointerEvent));
@@ -239,16 +306,29 @@ public:
                 pointerEvent = libinput_event_get_pointer_event(ev);
                 pointerScrollEvent.setDevice(inputDevice);
 
-                if (libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL))
+                pointerScrollEvent.setHasX(libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL) != 0);
+                pointerScrollEvent.setHasY(libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL) != 0);
+
+                if (pointerScrollEvent.hasX())
                 {
                     pointerScrollEvent.setX(libinput_event_pointer_get_scroll_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL));
-                    pointerScrollEvent.set120X(libinput_event_pointer_get_scroll_value_v120(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL));
+                    pointerScrollEvent.setDiscreteX(libinput_event_pointer_get_scroll_value_v120(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL));
+                }
+                else
+                {
+                    pointerScrollEvent.setX(0.f);
+                    pointerScrollEvent.setDiscreteX(0);
                 }
 
-                if (libinput_event_pointer_has_axis(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL))
+                if (pointerScrollEvent.hasY())
                 {
                     pointerScrollEvent.setY(libinput_event_pointer_get_scroll_value(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL));
-                    pointerScrollEvent.set120Y(libinput_event_pointer_get_scroll_value_v120(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL));
+                    pointerScrollEvent.setDiscreteY(libinput_event_pointer_get_scroll_value_v120(pointerEvent, LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL));
+                }
+                else
+                {
+                    pointerScrollEvent.setY(0.f);
+                    pointerScrollEvent.setDiscreteY(0);
                 }
 
                 pointerScrollEvent.setSource(LPointerScrollEvent::Wheel);
@@ -524,7 +604,13 @@ public:
             libinput_udev_assign_seat(li, "seat0");
 
         fd = libinput_get_fd(li);
-        eventSource = LCompositor::addFdListener(fd, (LSeat*)seat, &LInputBackend::processInput);
+
+        eventSource = wl_event_loop_add_fd(
+            LCompositor::eventLoop(),
+            fd,
+            WL_EVENT_READABLE,
+            &LInputBackend::processInput,
+            (LSeat*)seat);
         return true;
 
     fail:
@@ -536,7 +622,7 @@ public:
     {
         if (eventSource)
         {
-            LCompositor::removeFdListener(eventSource);
+            wl_event_source_remove(eventSource);
             eventSource = nullptr;
         }
 
