@@ -176,17 +176,19 @@ public:
 
         shared().fd[2].fd = eventfd(0, O_CLOEXEC | O_NONBLOCK);
 
-        eventfdEventSource = compositor()->addFdListener(
+        eventfdEventSource = wl_event_loop_add_fd(
+            LCompositor::eventLoop(),
             shared().fd[2].fd,
-            nullptr,
-            LInputBackend::processInput,
-            POLLIN);
+            WL_EVENT_READABLE,
+            &LInputBackend::processInput,
+            nullptr);
 
-        waylandEventSource = compositor()->addFdListener(
+        waylandEventSource = wl_event_loop_add_fd(
+            LCompositor::eventLoop(),
             shared().fd[1].fd,
-            nullptr,
-            LInputBackend::processInput,
-            POLLIN);
+            WL_EVENT_READABLE,
+            &LInputBackend::processInput,
+            nullptr);
 
         return true;
     }
@@ -195,14 +197,14 @@ public:
     {
         if (eventfdEventSource)
         {
-            compositor()->removeFdListener(eventfdEventSource);
+            wl_event_source_remove(eventfdEventSource);
             eventfdEventSource = nullptr;
             shared().fd[2].fd = -1;
         }
 
         if (waylandEventSource)
         {
-            compositor()->removeFdListener(waylandEventSource);
+            wl_event_source_remove(waylandEventSource);
             waylandEventSource = nullptr;
         }
 
@@ -437,15 +439,27 @@ public:
 
     static void pointerHandleAxis(void *, wl_pointer *, UInt32 time, UInt32 axis, Float24 value)
     {
-        Float32 val = wl_fixed_to_double(value);
+        const Float64 val { wl_fixed_to_double(value) };
+
+        pointerScrollEvent.setSource(LPointerScrollEvent::Continuous);
+        pointerScrollEvent.setDiscreteAxes(0, 0);
         pointerScrollEvent.setSerial(LTime::nextSerial());
         pointerScrollEvent.setMs(time);
         pointerScrollEvent.setUs(LTime::us());
 
+        pointerScrollEvent.setHasX(axis == WL_POINTER_AXIS_HORIZONTAL_SCROLL);
+        pointerScrollEvent.setHasY(axis == WL_POINTER_AXIS_VERTICAL_SCROLL);
+
         if (axis == WL_POINTER_AXIS_HORIZONTAL_SCROLL)
+        {
             pointerScrollEvent.setX(val);
+            pointerScrollEvent.setY(0.f);
+        }
         else
+        {
             pointerScrollEvent.setY(val);
+            pointerScrollEvent.setX(0.f);
+        }
 
         pointerScrollEvent.notify();
     }
