@@ -445,6 +445,24 @@ public:
      */
     const std::vector<Protocols::WlrOutputManagement::GWlrOutputManager*> &wlrOutputManagerGlobals() const noexcept;
 
+    /**
+     * Resources created when the client binds to the
+     * background_blur_manager global of the Background Blur protocol.
+     */
+    const std::vector<Protocols::BackgroundBlur::GBackgroundBlurManager*> &backgroundBlurManagerGlobals() const noexcept;
+
+    /**
+     * Resources created when the client binds to the
+     * svg_path_manager global of the SVG Path protocol.
+     */
+    const std::vector<Protocols::SvgPath::GSvgPathManager*> &svgPathManagerGlobals() const noexcept;
+
+    /**
+     * Resources created when the client binds to the
+     * invisible_region_manager global of the Invisible Region protocol.
+     */
+    const std::vector<Protocols::InvisibleRegion::GInvisibleRegionManager*> &invisibleRegionManagerGlobals() const noexcept;
+
     LPRIVATE_IMP_UNIQUE(LClient)
 };
 
