@@ -111,12 +111,15 @@ public:
     void finish() noexcept;
 
     /**
-     * @brief Gets the native `wl_display` used by the compositor.
-     *
-     * @return The native `wl_display`.
+     * @brief Gets the native `wl_display` created by the compositor.
      */
     static wl_display *display() noexcept;
 
+    /**
+     * @brief Gets the main event loop.
+     */
+    static wl_event_loop *eventLoop() noexcept;
+
     /**
      * @brief Adds a pollable file descriptor listener to the main event loop.
      *
@@ -125,10 +128,9 @@ public:
      * @param callback The callback function to handle events on the file descriptor.
      * @param flags Flags to specify the type of event to listen for (e.g., WL_EVENT_READABLE).
      *
-     * @note If the compositor is suspended, events are queued and will be dispatched once it is resumed.
-     *
      * @return The wl_event_source associated with the added file descriptor.
      */
+    [[deprecated("This function will be removed in Louvre v3.0. Use LCompositor::eventLoop() and wl_event_loop_add_fd instead.")]]
     static wl_event_source *addFdListener(int fd, void *userData, int(*callback)(int,unsigned int,void*), UInt32 flags = WL_EVENT_READABLE);
 
     /**
@@ -136,6 +138,7 @@ public:
      *
      * @param source The wl_event_source to remove.
      */
+    [[deprecated("This function will be removed in Louvre v3.0. Use wl_event_source_remove instead.")]]
     static void removeFdListener(wl_event_source *source);
 
     /**
@@ -436,15 +439,18 @@ public:
      * Louvre automatically creates all the supported globals during compositor initialization. See LCompositor::createGlobalsRequest().
      *
      * @tparam Global The type of global to create. It should be a subclass of Louvre::LResource.
+     * @param version The maximum supported interface version, constrained by Louvre’s implemented version.
+     *                If <= 0, Louvre’s highest supported version is used instead.
+     * @param data User data to assign to the `wl_resource`.
      * @return Pointer to the created LGlobal instance, which can be later removed with removeGlobal().
      */
     template<class Global>
-    LGlobal *createGlobal(void *data = nullptr) noexcept
+    LGlobal *createGlobal(Int32 version = 0, void *data = nullptr) noexcept
     {
         static_assert(std::is_base_of_v<Louvre::LResource, Global> == true);
 
         return globalCreate(Global::interface(),
-                            Global::maxVersion(),
+                            version <= 0 ? Global::maxVersion() : std::min(version, Global::maxVersion()),
                             data,
                             &Global::bind);
     }
