@@ -187,6 +187,14 @@ public:
      */
     LSurfaceLayer layer() const noexcept;
 
+    /**
+     * @brief Retrieves the background blur controller associated with the surface.
+     *
+     * @return A pointer to the background blur controller, guaranteed to exist even if
+     *         the client does not support the protocol.
+     */
+    LBackgroundBlur *backgroundBlur() const noexcept;
+
     /**
      * @brief Assigns the position.
      *
@@ -243,11 +251,15 @@ public:
      * @brief Input region in surface coordinates.
      *
      * Region of the surface that is capable of receiving input, in surface coordinates.
+     *
+     * Already clipped by the surface bounds.
      */
     const LRegion &inputRegion() const noexcept;
 
     /**
      * @brief Opaque region in surface coordinates.
+     *
+     * Already clipped by the surface bounds.
      */
     const LRegion &opaqueRegion() const noexcept;
 
@@ -255,16 +267,33 @@ public:
      * @brief Translucent region in surface coordinates.
      *
      * Translucent region in surface coordinates (inverted opaque region).
+     *
+     * Already clipped by the surface bounds.
      */
     const LRegion &translucentRegion() const noexcept;
 
+    /**
+     * @brief Invisible region in surface coordinates.
+     *
+     * Region within the surface that is completely transparent.
+     * 
+     * @note The invisible region protocol is experimental and thus not enabled by default in `LCompositor::createGlobalsRequest()`.
+     *
+     * Already clipped by the surface bounds.
+     */
+    const LRegion &invisibleRegion() const noexcept;
+
     /**
      * @brief Damaged region in surface coordinates.
+     *
+     * Already clipped by the surface bounds.
      */
     const LRegion &damage() const noexcept;
 
     /**
      * @brief Damaged region in buffer coordinates.
+     *
+     * Already clipped by the surface bounds.
      */
     const LRegion &damageB() const noexcept;
 
@@ -513,9 +542,6 @@ public:
     /**
      * @brief Check if the surface has a subchild popup.
      *
-     * This method checks whether the surface has a subchild popup surface.
-     * It returns `true` if the surface has a popup subchild; otherwise, it returns `false`.
-     *
      * @return `true` if the surface has a subchild popup, `false` otherwise.
      */
     bool hasPopupSubchild() const noexcept;
@@ -833,6 +859,16 @@ public:
      */
     virtual void opaqueRegionChanged();
 
+    /**
+     * @brief Notifies of a change in the invisible region
+     *
+     * Reimplement this virtual method if you wish to be notified when the surface changes its invisible region.
+     *
+     * #### Default Implementation
+     * @snippet LSurfaceDefault.cpp invisibleRegionChanged
+     */
+    virtual void invisibleRegionChanged();
+
     /**
      * @brief Notifies of a change in the input region
      *
