@@ -13,6 +13,7 @@
 #include <LActivationTokenManager.h>
 #include <LSessionLockManager.h>
 #include <LSessionLockRole.h>
+#include <LBackgroundBlur.h>
 #include <LAnimation.h>
 #include <LClipboard.h>
 #include <LKeyboard.h>
@@ -144,24 +145,24 @@ bool LCompositor::LCompositorPrivate::initWayland()
     }
 
     wl_display_init_shm(display);
-    waylandEventLoop = wl_display_get_event_loop(display);
-    auxEventLoop = wl_event_loop_create();
+    eventLoop = wl_display_get_event_loop(display);
+    eventLoopFd = wl_event_loop_get_fd(eventLoop);
 
-    compositor()->imp()->events[LEV_WAYLAND].events = EPOLLIN | EPOLLOUT;
-    compositor()->imp()->events[LEV_WAYLAND].data.fd = wl_event_loop_get_fd(waylandEventLoop);
+    // This is used to force unlocking the main loop with imp()->unlockPoll()
+    eventFd = eventfd(0, EFD_NONBLOCK);
 
-    epoll_ctl(compositor()->imp()->epollFd,
-              EPOLL_CTL_ADD,
-              compositor()->imp()->events[LEV_WAYLAND].data.fd,
-              &compositor()->imp()->events[LEV_WAYLAND]);
-
-    compositor()->imp()->events[LEV_AUX].events = EPOLLIN | EPOLLOUT;
-    compositor()->imp()->events[LEV_AUX].data.fd = wl_event_loop_get_fd(auxEventLoop);
-
-    epoll_ctl(compositor()->imp()->epollFd,
-              EPOLL_CTL_ADD,
-              compositor()->imp()->events[LEV_AUX].data.fd,
-              &compositor()->imp()->events[LEV_AUX]);
+    eventFdEventSource = wl_event_loop_add_fd(
+        LCompositor::eventLoop(),
+        eventFd,
+        WL_EVENT_READABLE,
+        [](int, unsigned int, void *) -> int {
+            UInt64 value;
+            auto ret = read(compositor()->imp()->eventFd, &value, sizeof(value));
+            L_UNUSED(ret)
+            compositor()->imp()->pollUnlocked = false;
+            return 0;
+        },
+        nullptr);
 
     // Listen for client connections
     clientConnectedListener.notify = &clientConnectedEvent;
@@ -198,14 +199,16 @@ bool LCompositor::LCompositorPrivate::initWayland()
 
 void LCompositor::LCompositorPrivate::unitWayland()
 {
-    if (auxEventLoop)
-    {
-        wl_event_loop_destroy(auxEventLoop);
-        auxEventLoop = nullptr;
-    }
-
     if (display)
-    {
+    {        
+        if (eventFdEventSource)
+        {
+            wl_event_source_remove(eventFdEventSource);
+            eventFdEventSource = nullptr;
+            close(eventFd);
+            eventFd = -1;
+        }
+
         while (!globals.empty())
         {
             delete globals.back();
@@ -213,6 +216,8 @@ void LCompositor::LCompositorPrivate::unitWayland()
         }
         wl_display_destroy(display);
         display = nullptr;
+        eventLoop = nullptr; // Destroyed by wl_display_destroy
+        eventLoopFd = -1;
     }
 
     if (activationTokenManager)
@@ -245,9 +250,6 @@ void LCompositor::LCompositorPrivate::unitCompositor()
 
     unitWayland();
 
-    if (epollFd != -1)
-        close(epollFd);
-
     while (!oneShotTimers.empty())
         delete oneShotTimers.back();
 
@@ -831,9 +833,9 @@ void LCompositor::LCompositorPrivate::unlockPoll()
         return;
 
     pollUnlocked = true;
-    uint64_t event_value = 1;
-    ssize_t n = write(events[0].data.fd, &event_value, sizeof(event_value));
-    L_UNUSED(n);
+    const uint64_t value { 1 };
+    auto ret = write(eventFd, &value, sizeof(value));
+    L_UNUSED(ret)
 }
 
 LPainter *LCompositor::LCompositorPrivate::findPainter()
@@ -850,6 +852,8 @@ void LCompositor::LCompositorPrivate::sendPendingConfigurations()
 {
     for (LSurface *s : surfaces)
     {
+        s->backgroundBlur()->sendPendingConfiguration();
+
         if (s->toplevel())
             s->toplevel()->sendPendingConfiguration();
         else if (s->popup())
@@ -1021,7 +1025,7 @@ void LCompositor::LCompositorPrivate::initDRMLeaseGlobals()
 {
     for (LGPU *gpu : *graphicBackend->backendGetDevices())
         if (gpu->fd() != -1 && gpu->roFd() != -1)
-            gpu->m_leaseGlobal.reset(compositor()->createGlobal<Protocols::DRMLease::GDRMLeaseDevice>(gpu));
+            gpu->m_leaseGlobal.reset(compositor()->createGlobal<Protocols::DRMLease::GDRMLeaseDevice>(0, gpu));
 }
 
 void LCompositor::LCompositorPrivate::unitDRMLeaseGlobals()
@@ -1131,7 +1135,7 @@ void LCompositor::LCompositorPrivate::handlePosixSignalChanges() noexcept
     while (!signalsCopy.empty())
     {
         LLog::debug("[LCompositorPrivate::handlePosixSignalChanges] Event source created for posix signal %d.", *signalsCopy.begin());
-        posixSignalSources[*signalsCopy.begin()] = wl_event_loop_add_signal(waylandEventLoop, *signalsCopy.begin(), &posixSignalHandler, nullptr);
+        posixSignalSources[*signalsCopy.begin()] = wl_event_loop_add_signal(eventLoop, *signalsCopy.begin(), &posixSignalHandler, nullptr);
         signalsCopy.erase(signalsCopy.begin());
     }
 }
