@@ -12,6 +12,7 @@
 #include <private/LFactory.h>
 #include <LCursorRole.h>
 #include <LDNDIconRole.h>
+#include <LBackgroundBlur.h>
 #include <LLog.h>
 
 using namespace Louvre::Protocols::Wayland;
@@ -95,6 +96,7 @@ RSurface::~RSurface()
 
     // Notify from client
     compositor()->onAnticipatedObjectDestruction(lSurface);
+    compositor()->onAnticipatedObjectDestruction(lSurface->imp()->backgroundBlur.get());
 
     // Unmap
     lSurface->imp()->setMapped(false);
@@ -331,7 +333,7 @@ void RSurface::apply_commit(LSurface *surface, LBaseSurfaceRole::CommitOrigin or
     {
         if (imp.stateFlags.check(LSurface::LSurfacePrivate::InfiniteInput))
         {
-            if (changes.check(Changes::SizeChanged))
+            if (changes.check(Changes::SizeChanged | Changes::InputRegionChanged))
             {
                 imp.currentInputRegion.clear();
                 imp.currentInputRegion.addRect(0, 0, surface->size());
@@ -379,6 +381,26 @@ void RSurface::apply_commit(LSurface *surface, LBaseSurfaceRole::CommitOrigin or
             imp.current.lockedPointerPosHint = imp.pending.lockedPointerPosHint;
     }
 
+    /****************************************
+     *********** INVISIBLE REGION ***********
+     ****************************************/
+    if (imp.stateFlags.check(LSurface::LSurfacePrivate::InfiniteInvisible))
+    {
+        if (changes.check(Changes::SizeChanged | Changes::InvisibleRegionChanged))
+        {
+            imp.currentInvisibleRegion.clear();
+            imp.currentInvisibleRegion.addRect(0, 0, surface->size());
+            changes.add(Changes::InvisibleRegionChanged);
+        }
+    }
+    else if (changes.check(Changes::SizeChanged | Changes::InvisibleRegionChanged))
+    {
+        pixman_region32_intersect_rect(&imp.currentInvisibleRegion.m_region,
+                                       &imp.pendingInvisibleRegion.m_region,
+                                       0, 0, surface->size().w(), surface->size().h());
+        changes.add(Changes::InvisibleRegionChanged);
+    }
+
     /************************************
      ********** OPAQUE REGION ***********
      ************************************/
@@ -465,12 +487,17 @@ void RSurface::apply_commit(LSurface *surface, LBaseSurfaceRole::CommitOrigin or
     if (changes.check(Changes::OpaqueRegionChanged))
         surface->opaqueRegionChanged();
 
+    if (changes.check(Changes::InvisibleRegionChanged))
+        surface->invisibleRegionChanged();
+
     if (changes.check(Changes::VSyncChanged))
         surface->preferVSyncChanged();
 
     if (changes.check(Changes::ContentTypeChanged))
         surface->contentTypeChanged();
 
+    surface->backgroundBlur()->handleCommit(changes.check(Changes::SizeChanged));
+
     changes.set(Changes::NoChanges);
 }
 
@@ -498,11 +525,15 @@ void RSurface::set_opaque_region(wl_client */*client*/, wl_resource *resource, w
     auto &imp { *static_cast<const RSurface*>(wl_resource_get_user_data(resource))->surface()->imp() };
 
     if (region)
+    {
         imp.pendingOpaqueRegion = static_cast<const RRegion*>(wl_resource_get_user_data(region))->region();
-    else
+        imp.changesToNotify.add(Changes::OpaqueRegionChanged);
+    }
+    else if (!imp.pendingOpaqueRegion.empty())
+    {
         imp.pendingOpaqueRegion.clear();
-
-    imp.changesToNotify.add(Changes::OpaqueRegionChanged);
+        imp.changesToNotify.add(Changes::OpaqueRegionChanged);
+    }
 }
 
 void RSurface::set_input_region(wl_client */*client*/, wl_resource *resource, wl_resource *region)
@@ -513,14 +544,14 @@ void RSurface::set_input_region(wl_client */*client*/, wl_resource *resource, wl
     {
         imp.pendingInputRegion = static_cast<const RRegion*>(wl_resource_get_user_data(region))->region();
         imp.stateFlags.remove(LSurface::LSurfacePrivate::InfiniteInput);
+        imp.changesToNotify.add(Changes::InputRegionChanged);
     }
-    else
+    else if (!imp.stateFlags.check(LSurface::LSurfacePrivate::InfiniteInput))
     {
         imp.pendingInputRegion.clear();
         imp.stateFlags.add(LSurface::LSurfacePrivate::InfiniteInput);
+        imp.changesToNotify.add(Changes::InputRegionChanged);
     }
-
-    imp.changesToNotify.add(Changes::InputRegionChanged);
 }
 
 #if LOUVRE_WL_COMPOSITOR_VERSION >= 2
